@@ -38,6 +38,8 @@ import time
 import uuid
 
 from timeline import Timeline
+from statusview import StatusView
+from timelinethread import BaseThread
 from twitterapi import TwitterAPI
 from iconstore import IconStore
 from saveconfig import save_configs, save_config, get_config, get_configs
@@ -48,16 +50,13 @@ import twittertools
 
 # Main Class
 class Main:
-    # init status timelines
-    timelines = list()
-    tlhash = dict()
-    
     # Default settings
-    interval = (60, 300, -1)
-    msgfooter = unicode()
+    interval = (300, 300, -1)
+    msgfooter = u""
     alloc = gtk.gdk.Rectangle(0, 0, 240, 320)
     scounts = (20, 200)
     iconmode = True
+    userstream = True
     # My status, Mentions to me, Reply to, Reply to user, Selected user
     status_color = ("#CCCCFF", "#FFCCCC", "#FFCC99", "#FFFFCC", "#CCFFCC")
     
@@ -75,10 +74,24 @@ class Main:
         gtk.gdk.threads_init()
         gobject.threads_init()
         
+        # init status timelines
+        self.timelines = list()
+        self.tlhash = dict()
+        
         # Twitter class instance
         self.twitter = TwitterAPI(screen_name, *keys)
         self.twitter.init_twitpic(self.twitpic_apikey)
         
+        # set event (show remaining api count)
+        self.twitter.on_twitterapi_requested = self.on_timeline_refresh
+        
+        # Get users
+        self.twitter.get_followers_bg()
+        if not self.userstream: self.twitter.get_following_bg()
+        
+        # init icon store
+        self.iconstore = IconStore(self.iconmode)
+        
         # GtkBuilder instance
         self.builder = gtk.Builder()
         # Glade file input
@@ -93,10 +106,19 @@ class Main:
         self.charcount = self.builder.get_object("label1")
         self.dsettings = self.builder.get_object("dialog_settings")
         
-        menu_tweet = self.builder.get_object("menu_tweet")
-        self.builder.get_object("menuitem_tweet").set_submenu(menu_tweet)
-        menu_timeline = self.builder.get_object("menu_timeline")
-        self.builder.get_object("menuitem_timeline").set_submenu(menu_timeline)
+        self.menu_tweet = self.builder.get_object("menu_tweet")
+        self.builder.get_object("menuitem_tweet").set_submenu(self.menu_tweet)
+        self.menu_timeline = self.builder.get_object("menu_timeline")
+        self.builder.get_object("menuitem_timeline").set_submenu(self.menu_timeline)
+        
+        # set class variables
+        Timeline.twitter = self.twitter
+        StatusView.twitter = self.twitter
+        StatusView.iconstore = self.iconstore
+        StatusView.iconmode = self.iconmode
+        BaseThread.twitter = self.twitter
+        StatusView.favico_y = self.notebook.render_icon("gtk-about", gtk.ICON_SIZE_MENU)
+        StatusView.favico_n = None
         
         # set tools
         self.twtools = twittertools.TwitterTools()
@@ -124,17 +146,15 @@ class Main:
             self.alloc = eval(d["allocation"])
             self.scounts = eval(d["counts"])
             self.iconmode = eval(d["iconmode"])
+            self.userstream = eval(d["userstream"])
             self.status_color = eval(d["color"])
             u = get_configs(self.twitter.myname)
             self.msgfooter = u["footer"]
         except Exception, e:
             print "[Error] Read settings: %s" % e
         
-        # init icon store
-        self.icons = IconStore(self.iconmode)
-        
         # Set Status Views
-        for i in (("Home", "home_timeline"),
+        for i in (("Home", "home_timeline", self.userstream),
                   ("Mentions", "mentions")):
             # create new timeline and tab view
             self.new_timeline(*i)
@@ -143,15 +163,15 @@ class Main:
         self.label_apilimit = gtk.Label()
         self.statusbar = self.builder.get_object("statusbar1")
         self.statusbar.pack_start(self.label_apilimit, expand = False, padding = 10)
-        self.statusbar.show_all()        
+        self.statusbar.show_all()
         
         # Users tab append
-        users = UserSelection(self.twitter, self.icons)
+        users = UserSelection(self.twitter, self.iconstore)
         users.new_timeline = self.new_timeline
         self.new_tab(users, "Users")
         
         # Lists tab append
-        lists = ListsSelection(self.twitter, self.icons)
+        lists = ListsSelection(self.twitter, self.iconstore)
         lists.new_timeline = self.new_timeline
         self.new_tab(lists, "Lists")
         
@@ -168,42 +188,44 @@ class Main:
         for i in self.timelines:
             if i != None:
                 i.destroy()
-                i.timeline.join()
+                if i.timeline != None: i.timeline.join(1)
+                if i.stream != None: i.stream.join(1)
         
         gtk.main_quit()
     
     # Create new Timeline and append to notebook
-    def new_timeline(self, label, method, *args, **kwargs):
+    def new_timeline(self, label, method, userstream = False, *args, **kwargs):
         # Create Timeline Object
-        tl = Timeline(self.twitter, self.icons, self.iconmode)
-        
         interval = self.get_default_interval(method)
-        
-        # Start sync timeline
-        tl.init_timeline(method, interval, self.scounts, args, kwargs)
-        tl.new_timeline = self.new_timeline
+        tl = Timeline()
+        tl.set_timeline(method, interval, self.scounts, args, kwargs)
+        tl.view.new_timeline = self.new_timeline
         
         # Add Notebook (Tab view)
-        self.new_tab(tl.scrwin, label, tl)
-
+        self.new_tab(tl, label, tl)
+        
        # Set color
-        tl.set_color(self.status_color)
+        tl.view.set_color(self.status_color)
         
         if method != "mentions":
-            tl.on_status_added = self.on_status_added
+            tl.view.on_status_added = self.on_status_added
         
-        # Set API Limit label
-        tl.timeline.on_timeline_refresh = self.on_timeline_refresh
         # Put error to statubar
         tl.timeline.on_twitterapi_error = self.on_twitterapi_error
         
         # Put tweet information to statusbar
-        tl.on_status_selection_changed = self.on_status_selection_changed
+        tl.view.on_status_selection_changed = self.on_status_selection_changed
         # Reply on double click
-        tl.on_status_activated = self.on_status_activated
+        tl.view.on_status_activated = self.on_status_activated
         
-        tl.start_timeline()        
-        tl.add_popup(self.builder.get_object("menu_tweet"))
+        tl.view.add_popup(self.menu_tweet)
+
+        # Set UserStream parameter
+        if userstream:
+            tl.set_stream("user")
+            tl.start_stream()
+        
+        tl.start_timeline()
     
     # Append Tab to Notebook
     def new_tab(self, widget, label, timeline = None):
@@ -237,11 +259,15 @@ class Main:
     
     def get_selected_status(self):
         n = self.notebook.get_current_page()
-        return self.timelines[n].get_selected_status()
+        if self.timelines[n] != None:
+            return self.timelines[n].view.get_selected_status()
     
-    def get_current_tab(self):
+    def get_current_tab_n(self):
         return self.notebook.get_current_page()
 
+    def get_current_tab(self):
+        return self.timelines[self.notebook.get_current_page()]
+    
     # Get text
     def get_textview(self):
         buf = self.textview.get_buffer()
@@ -267,7 +293,8 @@ class Main:
     # Reply to selected status
     def reply_to_selected_status(self):
         status = self.get_selected_status()
-
+        self.reply_to_status(status)
+    
     def reply_to_status(self, status):
         self.re = status.id
         name = status.user.screen_name
@@ -292,7 +319,7 @@ class Main:
         treeview.append_column(colt)
         
         status = self.twitter.statuses.values()[0]
-        store.append((self.icons.get(status.user),
+        store.append((self.iconstore.get(status.user),
                       "<b>%s</b>\n%s" % (status.user.screen_name, status.text),
                       color))
         
@@ -340,6 +367,30 @@ class Main:
         
         return color
     
+    def status_update_thread(self, status):
+        t = threading.Thread(target = self.status_update, args = (status,))
+        t.start()
+    
+    def status_update(self, status):
+        args = dict()
+        self.textview.set_sensitive(False)
+        self.btnupdate.set_sensitive(False)
+        
+        if self.re != None:
+            args["in_reply_to_status_id"] = self.re
+        elif self.msgfooter != "":
+            status = u"%s %s" % (status, self.msgfooter)
+        
+        resp = self.twitter.api_wrapper(self.twitter.api.status_update,
+                                        status, **args)
+        
+        if resp != None:
+            self.clear_textview()
+            self.re = None
+        
+        self.textview.set_sensitive(True)
+        self.btnupdate.set_sensitive(True)
+    
     def get_default_interval(self, method):
         if method == "home_timeline":
             interval = self.interval[0]
@@ -354,6 +405,7 @@ class Main:
         conf = (("DEFAULT", "interval", self.interval),
                 ("DEFAULT", "counts", self.scounts),
                 ("DEFAULT", "iconmode", self.iconmode),
+                ("DEFAULT", "userstream", self.userstream),
                 ("DEFAULT", "color", self.status_color),
                 (self.twitter.myname, "footer", self.msgfooter))
         save_configs(conf)
@@ -368,7 +420,7 @@ class Main:
         myname = self.twitter.myname
         if status.in_reply_to_screen_name == myname or \
                 status.text.find("@%s" % myname) >= 0:
-            self.timelines[1].timeline.add(set((status.id,)))
+            self.timelines[1].timeline.add_statuses(((status,)))
     
     # timeline refreshed event
     def on_timeline_refresh(self):
@@ -419,15 +471,13 @@ class Main:
         
         if txt != "":
             # Status Update
-            self.twitter.status_update(txt, self.re, self.msgfooter)
+            self.status_update(txt)
         else:
             # Reload timeline if nothing in textview
-            n = self.get_current_tab()
+            n = self.get_current_tab_n()
+            self.re = None
             if self.timelines[n] != None:
                 self.timelines[n].reload()
-        
-        self.re = None
-        self.clear_textview()
     
     # key_press textview (for update status when press Ctrl + Enter)
     def on_textview1_key_press_event(self, textview, event):
@@ -437,9 +487,7 @@ class Main:
             
             # if update button enabled (== len(text) <= 140
             if self.btnupdate.get_sensitive() and txt != "":
-                self.twitter.status_update(txt, self.re, self.msgfooter)
-                self.re = None
-                self.clear_textview()
+                self.status_update(txt)
             
             return True
     
@@ -484,7 +532,7 @@ class Main:
     # Tab right clicked
     def on_notebook_tabbar_button_press(self, widget, event):
         if event.button == 3:
-            self.builder.get_object("menu_timeline").popup(None, None, None, event.button, event.time)
+            self.menu_timeline.popup(None, None, None, event.button, event.time)
     
     # Character count
     def on_textbuffer1_changed(self, textbuffer):
@@ -512,14 +560,15 @@ class Main:
     # disable menu when switched tab
     def on_notebook1_switch_page(self, notebook, page, page_num):
         self.builder.get_object("menuitem_tweet").set_sensitive(False)
-        menu_timeline = self.builder.get_object("menuitem_timeline")
-        menu_timeline.set_sensitive(False)        
-        if page_num < 0: return
+        menuitem_timeline = self.builder.get_object("menuitem_timeline")
+        menuitem_timeline.set_sensitive(False)
+        if page_num < 0: return False
         
-        tl = self.timelines[page_num]
-        if tl != None:
+        tab = self.timelines[page_num]
+        if tab != None and tab.timeline != None:
             self._toggle_change_flg = True
-            method = tl.timeline.api_method.func_name
+            tl = tab.timeline
+            method = tl.method
             default = self.get_default_interval(method)
             
             if default == -1: default = None
@@ -527,7 +576,7 @@ class Main:
             menu_default = self.builder.get_object("menuitem_time_default")
             menu_default.get_child().set_text("Default (%s)" % default)
             
-            interval = tl.timeline.interval
+            interval = tl.interval
             
             if interval == default:
                 menu_default.set_active(True)
@@ -545,7 +594,35 @@ class Main:
                 self.builder.get_object("menuitem_time_30").set_active(True)
             
             self._toggle_change_flg = False
-            menu_timeline.set_sensitive(True)
+            menuitem_timeline.set_sensitive(True)
+    
+    # Streaming API tab
+    def on_menuitem_streaming_activate(self, menuitem):
+        dialog = gtk.MessageDialog(buttons = gtk.BUTTONS_OK)
+        dialog.set_markup("Please enter track keywords.")
+        dialog.format_secondary_markup("Examples: <i>hashtag, username, keyword</i>\n(Split comma. Unnecessary #, @)")
+        entry = gtk.Entry()
+        dialog.vbox.pack_start(entry)
+        dialog.show_all()
+        dialog.run()
+        text = entry.get_text()
+        dialog.destroy()
+        
+        params = {"track" : text.split(",")}
+        
+        tl = Timeline()
+        tl.set_stream("filter", params)
+        tl.view.new_timeline = self.new_timeline
+        
+        self.new_tab(tl, "Stream", tl)
+        tl.view.set_color(self.status_color)
+        tl.view.add_popup(self.menu_tweet)
+        tl.view.on_status_selection_changed = self.on_status_selection_changed
+        tl.view.on_status_activated = self.on_status_activated
+        
+        tl.view.add_popup(self.menu_tweet)
+        tl.start_stream()
+    
     
     ########################################
     # Tweet menu event
@@ -576,13 +653,13 @@ class Main:
     # Status detail
     def on_menuitem_detail_activate(self, menuitem):
         status = self.get_selected_status()
-        detail = StatusDetail(status, self.twitter, self.icons)
+        detail = StatusDetail(status, self.twitter, self.iconstore, self.iconmode)
+        #detail.view.add_popup(self.menu_tweet)
         self.new_tab(detail, "S: %d" % status.id)
     
     # favorite
     def on_menuitem_fav_activate(self, menuitem):
-        status = self.get_selected_status()
-        self.twitter.api_wrapper(self.twitter.api.favorite_create, status.id)
+        self.get_current_tab().favorite_selected_status()
     
     # Destroy status
     def on_menuitem_destroy_activate(self, menuitem):
@@ -595,7 +672,7 @@ class Main:
     def change_interval(self, interval):
         if self._toggle_change_flg: return
         
-        tl = self.timelines[self.get_current_tab()].timeline
+        tl = self.get_current_tab().timeline
         
         if interval == 0:
             method = tl.api_method.func_name
@@ -650,6 +727,8 @@ class Main:
         self.builder.get_object("spinbutton_maxn").set_value(self.scounts[1])
         # show icons
         self.builder.get_object("checkbutton_showicon").set_active(self.iconmode)
+        # userstream
+        self.builder.get_object("checkbutton_userstream").set_active(self.userstream)
         
         # footer
         self.builder.get_object("entry_footer").set_text(self.msgfooter)
@@ -704,6 +783,9 @@ class Main:
         # show icons
         self.iconmode = self.builder.get_object("checkbutton_showicon").get_active()
         
+        # userstream
+        self.userstream = self.builder.get_object("checkbutton_userstream").get_active()
+        
         # footer
         self.msgfooter = unicode(self.builder.get_object("entry_footer").get_text())
         
@@ -715,7 +797,7 @@ class Main:
                              self.builder.get_object("entry_color_selected_user").get_text())
         for t in self.timelines:
             if t != None:
-                t.set_color(self.status_color)
+                t.view.set_color(self.status_color)
         
         self.save_settings()
         self.dsettings.hide()
